@@ -1,5 +1,6 @@
 import time
 import json
+import logging
 from logging import Logger
 from typing import ClassVar, Iterator, Optional
 
@@ -15,9 +16,12 @@ from ..tap import SourceTap, SourceSettings, DestinationTap, DestinationSettings
 from ..message import MessageBase
 
 
+pipelineLogger = logging.getLogger("pipeline")
+
+
 class KafkaSourceSettings(SourceSettings):
     kafka: str = Field("localhost", title="kafka url")
-    group_id: str = Field(None, title="kafka consumer group id")
+    group_id: str = Field(..., title="kafka consumer group id")
     config: Optional[str] = Field(None, title="kafka config in json format")
     poll_timeout: int = Field(30, title="time out for polling new messages")
 
@@ -35,7 +39,9 @@ class KafkaSource(SourceTap):
 
     kind: ClassVar[str] = "KAFKA"
 
-    def __init__(self, settings: KafkaSourceSettings, logger: Logger) -> None:
+    def __init__(
+        self, settings: KafkaSourceSettings, logger: Logger = pipelineLogger
+    ) -> None:
         super().__init__(settings, logger)
 
         config = {
@@ -73,6 +79,9 @@ class KafkaSource(SourceTap):
             self.settings.kafka, self.settings.group_id, self.topic
         )
 
+    def __len__(self) -> int:
+        return -1
+
     def read(self) -> Iterator[MessageBase]:
         # sometimes it is better to make constant call to KAFKA
         # to keep the connection alive.
@@ -146,6 +155,9 @@ class KafkaDestination(DestinationTap):
             self.settings.kafka, self.topic
         )
 
+    def __len__(self) -> int:
+        return -1
+
     def write(self, message: MessageBase) -> int:
         def delivery_report(err, msg):  # type: ignore
             if err:
