@@ -11,16 +11,13 @@ from ..tap import SourceTap, SourceSettings, DestinationTap, DestinationSettings
 from ..message import MessageBase
 
 
-FORMAT = "%(asctime)-15s %(levelname)s %(message)s"
-logging.basicConfig(format=FORMAT)
 pipelineLogger = logging.getLogger("pipeline")
-pipelineLogger.setLevel(logging.DEBUG)
 
 
 class MongodbSourceSettings(SourceSettings):
     uri: str = Field("mongodb://localhost:27017", title="MongoDB url")
-    database: str = Field("", title="MongoDB database")
-    topic: str = Field("", title="collection name")
+    database: str = Field(..., title="MongoDB database")
+    topic: str = Field(..., title="collection name")
     keyname: str = Field("id", title="keyname")
     query: Json = Field("{}", title="query as json string")
 
@@ -50,6 +47,9 @@ class MongodbSource(SourceTap):
     def __repr__(self) -> str:
         return f'MongodbSource(host="{self.settings.uri}", topic="{self.topic}")'
 
+    def __len__(self) -> int:
+        return -1
+
     def read(self) -> Iterator[MessageBase]:
         for content in self.collection.find(self.query):
             yield MessageBase(
@@ -67,8 +67,8 @@ class MongodbSource(SourceTap):
 
 class MongodbDestinationSettings(DestinationSettings):
     uri: str = Field("mongodb://localhost:27017", title="MongoDB url")
-    database: str = Field("", title="MongoDB database")
-    topic: str = Field("", title="collection name")
+    database: str = Field(..., title="MongoDB database")
+    topic: str = Field(..., title="collection name")
     keyname: str = Field("id", title="keyname")
 
 
@@ -102,6 +102,9 @@ class MongodbDestination(DestinationTap):
     def __repr__(self) -> str:
         return f'MongdbDestination(host="{self.settings.uri}", topic="{self.topic}")'
 
+    def __len__(self) -> int:
+        return -1
+
     def get_filter_keyname(self, dct):
         return {self.keyname: dct.get(self.keyname)}
 
