@@ -1,4 +1,5 @@
 import time
+import logging
 from logging import Logger
 from typing import ClassVar, Iterator
 
@@ -6,10 +7,13 @@ import pika
 from pydantic import AnyUrl, Field
 
 from ..tap import SourceTap, SourceSettings, DestinationTap, DestinationSettings
-from ..message import Message
+from ..message import MessageBase
 from ..helpers import namespaced_topic
 
 
+pipelineLogger = logging.getLogger("pipeline")
+
+
 class RabbitMQDsn(AnyUrl):
     allowed_schemes = {
         "amqp",
@@ -43,7 +47,9 @@ class RabbitMQSource(SourceTap):
 
     kind: ClassVar[str] = "RABBITMQ"
 
-    def __init__(self, settings: RabbitMQSourceSettings, logger: Logger) -> None:
+    def __init__(
+        self, settings: RabbitMQSourceSettings, logger: Logger = pipelineLogger
+    ) -> None:
         super().__init__(settings, logger)
         self.settings = settings
         self.topic = settings.topic
@@ -60,9 +66,12 @@ class RabbitMQSource(SourceTap):
     def __repr__(self) -> str:
         return f'RabbitMQSource(queue="{self.name}")'
 
-    def read(self) -> Iterator[Message]:
+    def __len__(self) -> int:
+        return -1
+
+    def read(self) -> Iterator[MessageBase]:
         timedOut = False
-        lastMessageTime = time.time()
+        lastMessageBaseTime = time.time()
 
         while not timedOut:
             try:
@@ -81,10 +90,10 @@ class RabbitMQSource(SourceTap):
             if method:
                 self.delivery_tag = method.delivery_tag
                 self.logger.info("Read message %s", self.delivery_tag)
-                yield Message.deserialize(body)
-                lastMessageTime = time.time()
+                yield MessageBase.deserialize(body)
+                lastMessageBaseTime = time.time()
             time.sleep(0.01)
-            if self.timeout > 0 and time.time() - lastMessageTime > self.timeout:
+            if self.timeout > 0 and time.time() - lastMessageBaseTime > self.timeout:
                 self.logger.info("RabbitMQSource timed out.")
                 timedOut = True
 
@@ -138,7 +147,10 @@ class RabbitMQDestination(DestinationTap):
     def __repr__(self) -> str:
         return f'RabbitMQDestination(queue="{self.name}")'
 
-    def write(self, message: Message) -> int:
+    def __len__(self) -> int:
+        return -1
+
+    def write(self, message: MessageBase) -> int:
         serialized = message.serialize()
         try:
             self.channel.basic_publish(
