@@ -3,7 +3,7 @@ import csv
 import logging
 import sys
 import gzip
-import json
+import orjson
 from abc import ABC, abstractmethod
 from logging import Logger
 from enum import Enum
@@ -16,20 +16,24 @@ from .message import MessageBase, Message, deserialize_message, serialize_messag
 from .importor import import_class
 
 
-FORMAT = "%(asctime)-15s %(levelname)s %(message)s"
-logging.basicConfig(format=FORMAT)
 pipelineLogger = logging.getLogger("pipeline")
-pipelineLogger.setLevel(logging.DEBUG)
 
 
-class SourceSettings(Settings):
+class BaseSourceSettings(Settings):
+    class Config:
+        env_prefix = "in_"
+
+
+class BaseDestinationSettings(Settings):
+    class Config:
+        env_prefix = "out_"
+
+
+class SourceSettings(BaseSourceSettings):
     namespace: str = Field(None, title="source namespace")
     topic: str = Field("in-topic", title="source topic")
     timeout: int = Field(0, title="seconds to time out")
 
-    class Config:
-        env_prefix = "in_"
-
 
 class SourceTap(ABC):
     """Tap defines the interface for connecting components in pipeline.
@@ -49,6 +53,9 @@ class SourceTap(ABC):
     def __repr__(self) -> str:
         return f"Source(topic=({self.topic}), settings=({self.settings}))"
 
+    def __len__(self) -> int:
+        raise NotImplementedError()
+
     @abstractmethod
     def read(self) -> Iterator[MessageBase]:
         """receive message."""
@@ -90,14 +97,11 @@ class SourceTap(ABC):
         pass
 
 
-class DestinationSettings(Settings):
+class DestinationSettings(BaseDestinationSettings):
     namespace: str = Field(None, title="destination namespace")
     topic: str = Field("out-topic", title="output topic")
     compress: bool = Field(False, title="turn on compression")
 
-    class Config:
-        env_prefix = "out_"
-
 
 class DestinationTap(ABC):
     """Tap defines the interface for connecting components in pipeline."""
@@ -113,6 +117,9 @@ class DestinationTap(ABC):
     def __repr__(self) -> str:
         return f"Destination(topic=({self.topic}), settings=({self.settings}))"
 
+    def __len__(self) -> int:
+        raise NotImplementedError()
+
     @abstractmethod
     def write(self, message: MessageBase) -> int:
         """send message."""
@@ -167,6 +174,9 @@ class MemorySource(SourceTap):
 
         self.reset()
 
+    def __len__(self):
+        return len(self.data)
+
     def reset(self):
         self.load_data(self.settings.data)
 
@@ -208,6 +218,9 @@ class MemoryDestination(DestinationTap):
         self.results: List[MessageBase] = []
         self.storage: List[bytes] = []
 
+    def __len__(self):
+        return len(self.results)
+
     def reset(self):
         self.results = []
         self.storage = []
@@ -219,10 +232,8 @@ class MemoryDestination(DestinationTap):
 
 
 class FileSourceSettings(SourceSettings):
-    filename: str = Field(
-        None, title="input filename, use '-' for stdin", required=True
-    )
-    content_only: bool = Field(False, title="input contains only content for messages")
+    filename: str = Field("-", title="input filename, use '-' for stdin", required=True)
+    content_only: bool = Field(True, title="input contains only content for messages")
 
 
 class FileSource(SourceTap):
@@ -234,7 +245,7 @@ class FileSource(SourceTap):
     >>> with tempfile.NamedTemporaryFile() as tmpfile:
     ...     tmpfile.write(Message(content={"id": 0}).serialize()) and True
     ...     tmpfile.flush()
-    ...     settings = FileSourceSettings(filename=tmpfile.name)
+    ...     settings = FileSourceSettings(filename=tmpfile.name, content_only=False)
     ...     fileSource = FileSource(settings)
     ...     [m.content["id"] for m in fileSource.read()]
     True
@@ -259,10 +270,13 @@ class FileSource(SourceTap):
     def __repr__(self) -> str:
         return 'FileSource("{}")'.format(self.filename)
 
+    def __len__(self) -> int:
+        return -1
+
     def read(self) -> Iterator[MessageBase]:
         if self.settings.content_only:
             for line in self.infile:
-                content = json.loads(line)
+                content = orjson.loads(line)
                 if content.get("id"):
                     yield Message(content=content, id=content.get("id"))
                 else:
@@ -273,9 +287,9 @@ class FileSource(SourceTap):
 
 
 class FileDestinationSettings(DestinationSettings):
-    filename: str = Field(None, title="output filename", required=True)
+    filename: str = Field("-", title="output filename", required=True)
     overwrite: bool = Field(False, title="overwrite output file if exists")
-    content_only: bool = Field(False, title="output content only, no messages")
+    content_only: bool = Field(True, title="output content only, no messages")
 
 
 class FileDestination(DestinationTap):
@@ -294,25 +308,32 @@ class FileDestination(DestinationTap):
         super().__init__(settings, logger)
         self.filename = settings.filename
         if self.filename == "-":
+            self.logger.info("File Destination: stdout")
             self.outFile = sys.stdout.buffer
         elif self.filename.endswith(".gz"):
             if settings.overwrite:
+                self.logger.info("File Destination: %s (overwriting)", self.filename)
                 self.outFile = gzip.GzipFile(self.filename, "wb")  # type: ignore
             else:
+                self.logger.info("File Destination: %s", self.filename)
                 self.outFile = gzip.GzipFile(self.filename, "ab")  # type: ignore
         else:
             if settings.overwrite:
+                self.logger.info("File Destination: %s (overwriting)", self.filename)
                 self.outFile = open(self.filename, "wb")
             else:
+                self.logger.info("File Destination: %s", self.filename)
                 self.outFile = open(self.filename, "ab")
-        self.logger.info("File Destination: %s", self.filename)
 
     def __repr__(self) -> str:
         return 'FileDestination("{}")'.format(self.filename)
 
+    def __len__(self) -> int:
+        return -1
+
     def write(self, message: MessageBase) -> int:
         if self.settings.content_only:
-            serialized = (json.dumps(message.content) + "\n").encode("utf-8")
+            serialized = orjson.dumps(message.content) + b"\n"
         else:
             serialized = message.serialize(compress=False) + "\n".encode("utf-8")
         self.outFile.write(serialized)
@@ -370,6 +391,9 @@ class CsvSource(SourceTap):
     def __repr__(self) -> str:
         return 'CsvSource("{}")'.format(self.filename)
 
+    def __len__(self) -> int:
+        return -1
+
     def read(self) -> Iterator[MessageBase]:
         for row in self.reader:
             yield Message(content=row)
@@ -413,6 +437,9 @@ class CsvDestination(DestinationTap):
     def __repr__(self) -> str:
         return 'CsvDestination("{}")'.format(self.filename)
 
+    def __len__(self) -> int:
+        return -1
+
     def write(self, message: MessageBase) -> int:
         if self.writer is None:
             self.writer = csv.DictWriter(
@@ -438,12 +465,12 @@ class TapAndSettingsImportStrings(BaseModel):
 
 class SourceAndSettingsClasses(BaseModel):
     source_class: Type[SourceTap]
-    settings_class: Type[SourceSettings]
+    settings_class: Type[BaseSourceSettings]
 
 
 class DestinationAndSettingsClasses(BaseModel):
     destination_class: Type[DestinationTap]
-    settings_class: Type[DestinationSettings]
+    settings_class: Type[BaseDestinationSettings]
 
 
 def tap_kinds() -> Dict[
@@ -483,21 +510,21 @@ def tap_kinds() -> Dict[
         "XREDIS": (
             TapAndSettingsImportStrings(
                 tap_class="pipeline.backends.redis:RedisStreamSource",
-                settings_class="pipeline.backends.redis:RedisSourceSettings",
+                settings_class="pipeline.backends.redis:RedisStreamSourceSettings",
             ),
             TapAndSettingsImportStrings(
                 tap_class="pipeline.backends.redis:RedisStreamDestination",
-                settings_class="pipeline.backends.redis:RedisDestinationSettings",
+                settings_class="pipeline.backends.redis:RedisStreamDestinationSettings",
             ),
         ),
         "LREDIS": (
             TapAndSettingsImportStrings(
                 tap_class="pipeline.backends.redis:RedisListSource",
-                settings_class="pipeline.backends.redis:RedisSourceSettings",
+                settings_class="pipeline.backends.redis:RedisListSourceSettings",
             ),
             TapAndSettingsImportStrings(
                 tap_class="pipeline.backends.redis:RedisListDestination",
-                settings_class="pipeline.backends.redis:RedisDestinationSettings",
+                settings_class="pipeline.backends.redis:RedisListDestinationSettings",
             ),
         ),
         "KAFKA": (
@@ -550,6 +577,16 @@ def tap_kinds() -> Dict[
                 settings_class="pipeline.backends.mongodb:MongodbDestinationSettings",
             ),
         ),
+        "RQ": (
+            TapAndSettingsImportStrings(
+                tap_class="pipeline.backends.rq:RQSource",
+                settings_class="pipeline.backends.rq:RQSourceSettings",
+            ),
+            TapAndSettingsImportStrings(
+                tap_class="pipeline.backends.rq:RQDestination",
+                settings_class="pipeline.backends.rq:RQDestinationSettings",
+            ),
+        ),
     }
 
 
