@@ -6,7 +6,18 @@ from argparse import ArgumentParser
 from copy import copy
 from datetime import datetime
 from enum import IntEnum, Enum
-from typing import Optional, List, Dict, Iterator, Type, KeysView, Union, cast, Any
+from typing import (
+    Optional,
+    List,
+    Dict,
+    Iterator,
+    Type,
+    KeysView,
+    Union,
+    cast,
+    Any,
+    Tuple,
+)
 from logging import Logger
 
 from pydantic import BaseModel, ByteSize, Field, ValidationError, parse_obj_as
@@ -137,8 +148,9 @@ class Worker(ABC):
 
         if self.has_input() and self.settings.in_kind:
             self.source_and_settings_classes = SourceTap.of(self.settings.in_kind)
-            source_settings = self.source_and_settings_classes.settings_class()
-            source_settings.parse_args(args, parser)
+            source_settings = self.source_and_settings_classes.settings_class(
+                _args=args, _parser=parser
+            )
             if not options.help:
                 self.source = self.source_and_settings_classes.source_class(
                     settings=source_settings, logger=self.logger
@@ -149,10 +161,9 @@ class Worker(ABC):
             self.destination_and_settings_classes = DestinationTap.of(
                 self.settings.out_kind
             )
-            destination_settings = (
-                self.destination_and_settings_classes.settings_class()
+            destination_settings = self.destination_and_settings_classes.settings_class(
+                _args=args, _parser=parser
             )
-            destination_settings.parse_args(args, parser)
             if not options.help:
                 self.destination = (
                     self.destination_and_settings_classes.destination_class(
@@ -179,6 +190,19 @@ class Worker(ABC):
         # report worker info to monitor
         self.monitor.record_worker_info()
 
+    def multiple_destinations(self):
+        self.destinations: Dict[str, DestinationTap] = {}
+
+    def duplicate_destination(self, topic):
+        """helper function"""
+        settings = copy(self.destination.settings)
+        settings.topic = topic
+        self.destinations[
+            topic
+        ] = self.destination_and_settings_classes.destination_class(
+            settings, logger=self.logger
+        )
+
 
 class ProducerSettings(WorkerSettings):
     """Producer settings"""
@@ -228,8 +252,21 @@ class Producer(Worker):
         """a producer to generate dict."""
         yield self.output_class()
 
-    def _step(self) -> BaseModel:
-        return next(self.generator)
+    def make_message(self, content: BaseModel) -> Message:
+        if hasattr(content, "id"):
+            msg = Message(id=content.id, content=content.dict())  # type: ignore
+        else:
+            msg = Message(content=content.dict())
+        return msg
+
+    def step(self) -> Tuple[int, Message]:
+        """make new message and write to destination
+        rtype: Tuple[int, Message]
+        """
+        output = next(self.generator)
+        msg = self.make_message(output)
+        size = self.destination.write(msg)
+        return size, msg
 
     def start(self) -> None:
         try:
@@ -264,19 +301,14 @@ class Producer(Worker):
                     received=datetime.now(),
                 )
                 self.timer.start()
-                output = self._step()
+                size, msg = self.step()
                 self.timer.log(self.logger)
-                if hasattr(output, "id"):
-                    msg = Message(id=output.id, content=output.dict())  # type: ignore
-                else:
-                    msg = Message(content=output.dict())
-                self.logger.info("Generated %d-th message %s", i, msg)
-                log.updated.update(output.dict().keys())
-                msg.logs.append(log)
-                self.logger.info("Writing message %s", msg)
-                size = ByteSize(self.destination.write(msg))
-                self.logger.info(f"Message size: {size.human_readable()}")
+                self.logger.info("Generated %d-th message message %s", i, msg)
+                log.updated.update(msg.content.keys())
+                human_readable_size = ByteSize(size).human_readable()
+                self.logger.info(f"Message size: {human_readable_size}")
                 self.monitor.record_write(self.destination.topic)
+
         except StopIteration:
             pass
         except PipelineError as e:
@@ -308,7 +340,7 @@ class Splitter(Worker):
         super().__init__(settings, logger=logger)
         # keep a dictionary for 'topic': 'destination', self.destination is only used to parse
         # command line arguments
-        self.destinations: Dict[str, DestinationTap] = {}
+        self.multiple_destinations()
 
     def get_topic(self, msg: Message) -> str:
         raise NotImplementedError("You need to implement .get_topic(self, msg)")
@@ -329,13 +361,7 @@ class Splitter(Worker):
                 topic = self.get_topic(cast(Message, msg))
 
                 if topic not in self.destinations:
-                    settings = copy(self.destination.settings)
-                    settings.topic = topic
-                    self.destinations[
-                        topic
-                    ] = self.destination_and_settings_classes.destination_class(
-                        settings, logger=self.logger
-                    )
+                    self.duplicate_destination(topic)
 
                 destination = self.destinations[topic]
 
@@ -574,6 +600,7 @@ class Processor(Worker):
             self.timer.log(self.logger)
 
             self.source.acknowledge()
+            self.logger.info("Acknowledged %d-th message", i)
 
             if i == self.limit:
                 self.logger.info(f"Limit {self.limit} reached, exiting")
